#include "GHZ_SPI.h"
#include "delay.h"
#include "discover_board.h"


// Framer initalization registers
// {Adress, DataH, DataL}
const uint8_t rf_fram_table[][3]={
   {0x30,0x98,0x00},{0x31,0xff,0x0F},{0x32,0x80,0x28},{0x33,0x80,0x56},	
   {0x34,0x4E,0xF6},{0x35,0xF6,0xF5},{0x36,0x18,0x5C},{0x37,0xD6,0x51},
   {0x38,0x44,0x04},{0x39,0xE0,0x00},{0x3a,0x00,0x00}
}; 

// RFIC initalization registers
const uint8_t rf_regi_table[][3]={
   {0x09,0x00,0x01}, {0x00,0x35,0x4F},
   {0x02,0X1F,0X01}, {0x04,0x38,0xD8},
   {0x05,0x00,0xA1},{0x07,0x12,0x4C},{0x08,0x80,0x08},{0x0a,0x00,0x04},
   {0x0b,0X40,0X41},{0x0c,0X7E,0X00},{0x0d,0X00,0X00},{0x0e,0x16,0x9d},
   {0x0f,0x80,0x2f},{0x10,0xb0,0xf8},{0x12,0xe0,0x00},{0x13,0xa1,0x14},
   {0x14,0x81,0x91},{0x15,0x69,0x62},{0x16,0x00,0x02},{0x17,0x00,0x02},
   {0x18,0xb1,0x40},{0x19,0x78,0x0f},{0x1a,0x3f,0x04},{0x1c,0x58,0x00},
};


uint8_t SPI_TxRx8(SPI_TypeDef* SPIx, uint8_t Data)
{
  uint8_t SPI_Read;
SPIx->DR = Data; /* Write in the DR register the data to be sent*/
__asm("nop\n");
__asm("nop\n");
while (SPI_GetFlagStatus(SPI1,SPI_FLAG_TXE)==RESET);
while (!SPI_GetFlagStatus(SPI1,SPI_FLAG_RXNE));
SPI_Read=SPIx->DR;
/* while (SPI_GetFlagStatus(SPI1,SPI_FLAG_BSY)==SET);*/
return SPI_Read; /* Return the data in the DR register*/
}

uint16_t GHZ_ReadReg(SPI_TypeDef* SPIx, uint8_t RegAddr)
{
uint8_t dataH, dataL;

GPIO_LOW(SPI_SEL_PORT, GHZ_SEL_PIN);      
SPI_TxRx8(SPIx,(RegAddr|0x80));
dataH=SPI_TxRx8(SPIx,0xFF); 
dataL=SPI_TxRx8(SPIx,0xFF); 
GPIO_HIGH(SPI_SEL_PORT, GHZ_SEL_PIN);    
return (uint16_t)((dataH<<8)|(dataL));  
}

void GHZ_WriteReg(SPI_TypeDef* SPIx, uint8_t RegAddr, uint16_t RegData)
{
GPIO_LOW(SPI_SEL_PORT, GHZ_SEL_PIN);      
asm("nop");
SPI_TxRx8(SPIx,(RegAddr & 0x7F));
asm("nop");
SPI_TxRx8(SPIx,(uint8_t)(RegData >> 8));  
asm("nop");
SPI_TxRx8(SPIx,(uint8_t)(0x00FF&RegData));   
asm("nop");
GPIO_HIGH(SPI_SEL_PORT, GHZ_SEL_PIN);
asm("nop");
asm("nop");
asm("nop");
asm("nop");
}

void GHZ_Init(void)
{
int i; 
 
for ( i=0; i<sizeof(rf_fram_table)/sizeof(*rf_fram_table); i++ )
   {
      GHZ_WriteReg(SPI1, (rf_fram_table[i][0]),(uint16_t)(((rf_fram_table[i][1])<<8)|(rf_fram_table[i][2])));
      delay_10us(2); }
      
  delay_ms(100); // wait for RFIC calibration


   // Test for critical error. Possible reason of this error is bad layout of
   // transceiver board
  if ( (GHZ_ReadReg(SPI1,0x40)>>8) != 0xc0 ) while(1);  
   //INITIAL REG0~REG28
   for (i=0;i<sizeof(rf_regi_table)/sizeof(*rf_regi_table);i++)
   {
      GHZ_WriteReg(SPI1,(rf_regi_table[i][0]),(uint16_t)(((rf_regi_table[i][1])<<8)|(rf_regi_table[i][2])));
      delay_10us(2);
   }
      
 
   
GHZ_WriteReg(SPI1,0x07, 0x0000);   //Go to idle mode

}


void GHZ_RcvMode (uint8_t Channel)
{
if (Channel > 127) Channel = 127;
  GHZ_WriteReg(SPI1,0x52,0x0080); //RESET RX FIFO
  GHZ_WriteReg(SPI1,0x07,(1<<7)|(Channel)); //Enable RX @ 2402+CH_NO
}